const Ci = Components.interfaces;
const Cc = Components.classes;
const Cr = Components.results;

const NS_ADBLOCKPP = "http://park2.wakwak.com/~benki/adblockpp#";
const RDF_ROOT = {
	filter:		"urn:filter:item:root",
	whitelist:	"urn:whitelist:item:root",
	popup:		"urn:popup:item:root"
};
const gPaneID = {
	filter:		"paneFilters",
	whitelist:	"paneWhiteLists",
	popup:		"panePopupFilters"
};

var gRDFService = Cc["@mozilla.org/rdf/rdf-service;1"]
				.getService(Ci.nsIRDFService);
var gRDFContainerUtils = Cc["@mozilla.org/rdf/container-utils;1"]
				.getService(Ci.nsIRDFContainerUtils);

var gElement;
var gID = 0;

var gDS = {
	filter:		Cc["@mozilla.org/rdf/datasource;1?name=in-memory-datasource"]
				.createInstance(Ci.nsIRDFDataSource),
	whitelist:	Cc["@mozilla.org/rdf/datasource;1?name=in-memory-datasource"]
				.createInstance(Ci.nsIRDFDataSource),
	popup:		Cc["@mozilla.org/rdf/datasource;1?name=in-memory-datasource"]
				.createInstance(Ci.nsIRDFDataSource)
};

var gRoot = {
	filter:		gRDFContainerUtils.MakeSeq(gDS.filter, gRDFService.GetResource(RDF_ROOT.filter)),
	whitelist:	gRDFContainerUtils.MakeSeq(gDS.whitelist, gRDFService.GetResource(RDF_ROOT.whitelist)),
	popup:		gRDFContainerUtils.MakeSeq(gDS.popup, gRDFService.GetResource(RDF_ROOT.popup))
};

const gProp = {
	
	filter: {
		string:	gRDFService.GetResource(NS_ADBLOCKPP+"filterString"),
	},
	
	whitelist: {
		string:	gRDFService.GetResource(NS_ADBLOCKPP+"whiteListString"),
	},
	
	popup: {
		string:	gRDFService.GetResource(NS_ADBLOCKPP+"popupString"),
	}
};

function init() {
	gElement = {
		filter: {
			tree:		document.getElementById("filterTree"),
			textbox:	document.getElementById("filterTextbox"),
		},
		whitelist: {
			tree:		document.getElementById("whiteListTree"),
			textbox:	document.getElementById("whiteListTextbox"),
		},
		popup: {
			tree:		document.getElementById("popupTree"),
			textbox:	document.getElementById("popupTextbox"),
		}
	};
	
	var pref = Cc["@mozilla.org/preferences-service;1"]
				.getService(Ci.nsIPrefService)
				.getBranch("");
	var array = pref.getCharPref("extensions.adblock++_lite.filter").split("<>");
	appendTree(array);
}

function finish() {
}

function update() {
	var filters = new Array();

	for (var i in gDS) {
		var resources = gDS[i].GetAllResources();
		while (resources.hasMoreElements()) {
			var resource = resources.getNext().QueryInterface(Ci.nsIRDFResource);
			
			var target = gDS[i].GetTarget(resource, gProp[i].string, true);
			if (!target) continue;
			var rawString = target.QueryInterface(Ci.nsIRDFLiteral).Value;
			
			if (i == "whitelist") {
				rawString = "@@" + rawString;
			} else if (i == "popup") {
				rawString = "$$" + rawString;
			}
			
			var filter = new Array(rawString);
			filters.push(filter);
		}
	}
	var pref = Cc["@mozilla.org/preferences-service;1"]
				.getService(Ci.nsIPrefService)
				.getBranch("");
	pref.setCharPref("extensions.adblock++_lite.filter", filters.join("<>"));
}

function appendTree(aArray) {
	
	for (var i in gDS) {
		gElement[i].tree.database.AddDataSource(gDS[i]);
		gElement[i].tree.setAttribute("ref", RDF_ROOT[i]);
	}
		
	for (var j=0; j<aArray.length; j++) {
		var filter = aArray[j];
		
		var type;
		if (filter.indexOf("@@") == 0) {
			filter = filter.substring(2);
			type = "whitelist";
		} else if (filter.indexOf("$$") == 0) {
			filter = filter.substring(2);
			type = "popup";
		} else {
			type = "filter";
		}
		
		var resource = gRDFService.GetResource(gID++);
		gRoot[type].AppendElement(resource);
		gDS[type].beginUpdateBatch();
		gDS[type].Assert(resource, gProp[type].string, gRDFService.GetLiteral(filter), true);
		gDS[type].endUpdateBatch();
	}
	
	for (i in gDS)
		gElement[i].tree.builder.rebuild();
}

function add() {
	for (var i in gPaneID) {
		if (document.getElementById("adblock++").currentPane.id == gPaneID[i]) {
			if (!gElement[i].textbox.value) return;
			
			var resource = gRDFService.GetResource(gID++);
			
			gDS[i].beginUpdateBatch();
			gRoot[i].AppendElement(resource);
			gDS[i].Assert(resource, gProp[i].string, gRDFService.GetLiteral(gElement[i].textbox.value), true);
			gDS[i].endUpdateBatch();
			
			gElement[i].tree.builder.rebuild();
			var index = gElement[i].tree.view.getIndexOfResource(resource);
			gElement[i].tree.view.selection.select(index);
			gElement[i].tree.treeBoxObject.ensureRowIsVisible(index);
		}
	}
}

function edit() {
	for (var i in gPaneID) {
		if (document.getElementById("adblock++").currentPane.id == gPaneID[i]) {
			if (!gElement[i].textbox.value) return;
			
			if (!gElement[i].tree.view.selection) return;
			
			var currentIndex = gElement[i].tree.view.selection.currentIndex;
			if (currentIndex < 0) return;
			
			var resource = gElement[i].tree.view.getResourceAtIndex(currentIndex);
			
			gDS[i].beginUpdateBatch();
			gDS[i].Change(resource, gProp[i].string, gDS[i].GetTarget(resource, gProp[i].string, true), gRDFService.GetLiteral(gElement[i].textbox.value));
			gDS[i].endUpdateBatch();
			
			gElement[i].tree.builder.rebuild();
			var index = gElement[i].tree.view.getIndexOfResource(resource);
			gElement[i].tree.view.selection.select(index);
			gElement[i].tree.treeBoxObject.ensureRowIsVisible(index);
		}
	}
}

function del() {
	for (var i in gPaneID) {
		if (document.getElementById("adblock++").currentPane.id == gPaneID[i]) {
			if (!gElement[i].tree.view.selection) return;
			
			var currentIndex = gElement[i].tree.view.selection.currentIndex;
			if (currentIndex < 0) return;
			
			var resource = gElement[i].tree.view.getResourceAtIndex(currentIndex);
			
			gDS[i].beginUpdateBatch();
			
			gRoot[i].RemoveElement(resource, true);
			
			var names = gDS[i].ArcLabelsOut(resource);
			var name, value;
			while (names.hasMoreElements()) {
				name = names.getNext().QueryInterface(Ci.nsIRDFResource);
				value = gDS[i].GetTarget(resource, name, true);
				gDS[i].Unassert(resource, name, value);
			}
			
			gDS[i].endUpdateBatch();
			
			var rowCount = gElement[i].tree.view.rowCount;
			if (currentIndex == rowCount)
				currentIndex--;
			
			gElement[i].tree.builder.rebuild();
			
			if (rowCount > 0) {
				gElement[i].tree.view.selection.select(currentIndex);
				gElement[i].tree.treeBoxObject.ensureRowIsVisible(currentIndex);
			} else
				gElement[i].textbox.focus();
		}
	}
}

function onSelect() {
	for (var i in gPaneID) {
		if (document.getElementById("adblock++").currentPane.id == gPaneID[i]) {
			if (gElement[i].tree.view.selection.currentIndex < 0) return;
			var resource = gElement[i].tree.view.getResourceAtIndex(gElement[i].tree.view.selection.currentIndex);
			
			gElement[i].textbox.value = gDS[i].GetTarget(resource, gProp[i].string, true).QueryInterface(Ci.nsIRDFLiteral).Value;
		}
	}
}
